package org.example;

import com.szkingdom.koca.logging.core.MultiAppLoggerSpaceManager;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.springframework.util.StopWatch;

@Aspect
public class TimeElapsedMonitorAop {

    private static final Logger LOGGER = MultiAppLoggerSpaceManager.getLoggerBySpace(TimeElapsedMonitorAop.class.getCanonicalName(), "koca-time-elapsed");

    @Pointcut("@annotation(org.springframework.web.bind.annotation.GetMapping)")
    public void pointcut(){}

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint point) {
        Signature signature = point.getSignature();
        String method = signature.toShortString();
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();

        try {
            return point.proceed();
        } catch (Throwable e) {
            stopWatch.stop();
            throw new RuntimeException(e);
        } finally {
            stopWatch.stop();
            LOGGER.info("API call elapsed[{}]: {}", stopWatch.getTotalTimeSeconds(), method);
        }
    }
}