package org.example;

import com.szkingdom.koca.logging.core.MultiAppLoggerSpaceManager;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;

@Aspect
public class TraceMonitorAop {

    private static final Logger LOGGER = MultiAppLoggerSpaceManager.getLoggerBySpace(TraceMonitorAop.class.getCanonicalName(), "koca-trace-log");

    @Pointcut("@annotation(org.springframework.web.bind.annotation.GetMapping)")
    public void pointcut(){}

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint point) {
        Signature signature = point.getSignature();
        String method = signature.toShortString();
        LOGGER.info("API called begin: {}", method);

        try {
            return point.proceed();
        } catch (Throwable e) {
            throw new RuntimeException(e);
        } finally {
            LOGGER.info("API call end: {}", method);
        }
    }
}